# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .ases import (
    AsesResource,
    AsyncAsesResource,
    AsesResourceWithRawResponse,
    AsyncAsesResourceWithRawResponse,
    AsesResourceWithStreamingResponse,
    AsyncAsesResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ......_utils import maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......_base_client import make_request_options
from ......types.radar.attacks.layer7 import top_attacks_params, top_industry_params, top_vertical_params
from ......types.radar.attacks.layer7.top_attacks_response import TopAttacksResponse
from ......types.radar.attacks.layer7.top_industry_response import TopIndustryResponse
from ......types.radar.attacks.layer7.top_vertical_response import TopVerticalResponse

__all__ = ["TopResource", "AsyncTopResource"]


class TopResource(SyncAPIResource):
    @cached_property
    def locations(self) -> LocationsResource:
        return LocationsResource(self._client)

    @cached_property
    def ases(self) -> AsesResource:
        return AsesResource(self._client)

    @cached_property
    def with_raw_response(self) -> TopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TopResourceWithStreamingResponse(self)

    def attacks(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        limit_direction: Literal["ORIGIN", "TARGET"] | Omit = omit,
        limit_per_location: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN_MAX"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopAttacksResponse:
        """Retrieves the top attacks from origin to target location.

        Values are percentages
        of the total layer 7 attacks (with billing country). The attack magnitude can be
        defined by the number of mitigated requests or by the number of zones affected.
        You can optionally limit the number of attacks by origin/target location (useful
        if all the top attacks are from or to the same location).

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          limit_direction: Specifies whether the `limitPerLocation` applies to the source or target
              location.

          limit_per_location: Limits the number of attacks per origin/target (refer to `limitDirection`
              parameter) location.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/attacks/layer7/top/attacks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "limit_direction": limit_direction,
                        "limit_per_location": limit_per_location,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                        "normalization": normalization,
                    },
                    top_attacks_params.TopAttacksParams,
                ),
                post_parser=ResultWrapper[TopAttacksResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopAttacksResponse], ResultWrapper[TopAttacksResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar Attacks Layer 7 Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/attacks/subresources/layer7/methods/summary_v2/) instead."
    )
    def industry(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        http_method: List[
            Literal[
                "GET",
                "POST",
                "DELETE",
                "PUT",
                "HEAD",
                "PURGE",
                "OPTIONS",
                "PROPFIND",
                "MKCOL",
                "PATCH",
                "ACL",
                "BCOPY",
                "BDELETE",
                "BMOVE",
                "BPROPFIND",
                "BPROPPATCH",
                "CHECKIN",
                "CHECKOUT",
                "CONNECT",
                "COPY",
                "LABEL",
                "LOCK",
                "MERGE",
                "MKACTIVITY",
                "MKWORKSPACE",
                "MOVE",
                "NOTIFY",
                "ORDERPATCH",
                "POLL",
                "PROPPATCH",
                "REPORT",
                "SEARCH",
                "SUBSCRIBE",
                "TRACE",
                "UNCHECKOUT",
                "UNLOCK",
                "UNSUBSCRIBE",
                "UPDATE",
                "VERSIONCONTROL",
                "BASELINECONTROL",
                "XMSENUMATTS",
                "RPC_OUT_DATA",
                "RPC_IN_DATA",
                "JSON",
                "COOK",
                "TRACK",
            ]
        ]
        | Omit = omit,
        http_version: List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]] | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopIndustryResponse:
        """This endpoint is deprecated.

        To continue getting this data, switch to the
        summary by industry endpoint.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          http_method: Filters results by HTTP method.

          http_version: Filters results by HTTP version.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/attacks/layer7/top/industry",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "http_method": http_method,
                        "http_version": http_version,
                        "ip_version": ip_version,
                        "limit": limit,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                    },
                    top_industry_params.TopIndustryParams,
                ),
                post_parser=ResultWrapper[TopIndustryResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopIndustryResponse], ResultWrapper[TopIndustryResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar Attacks Layer 7 Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/attacks/subresources/layer7/methods/summary_v2/) instead."
    )
    def vertical(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        http_method: List[
            Literal[
                "GET",
                "POST",
                "DELETE",
                "PUT",
                "HEAD",
                "PURGE",
                "OPTIONS",
                "PROPFIND",
                "MKCOL",
                "PATCH",
                "ACL",
                "BCOPY",
                "BDELETE",
                "BMOVE",
                "BPROPFIND",
                "BPROPPATCH",
                "CHECKIN",
                "CHECKOUT",
                "CONNECT",
                "COPY",
                "LABEL",
                "LOCK",
                "MERGE",
                "MKACTIVITY",
                "MKWORKSPACE",
                "MOVE",
                "NOTIFY",
                "ORDERPATCH",
                "POLL",
                "PROPPATCH",
                "REPORT",
                "SEARCH",
                "SUBSCRIBE",
                "TRACE",
                "UNCHECKOUT",
                "UNLOCK",
                "UNSUBSCRIBE",
                "UPDATE",
                "VERSIONCONTROL",
                "BASELINECONTROL",
                "XMSENUMATTS",
                "RPC_OUT_DATA",
                "RPC_IN_DATA",
                "JSON",
                "COOK",
                "TRACK",
            ]
        ]
        | Omit = omit,
        http_version: List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]] | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopVerticalResponse:
        """This endpoint is deprecated.

        To continue getting this data, switch to the
        summary by vertical endpoint.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          http_method: Filters results by HTTP method.

          http_version: Filters results by HTTP version.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/attacks/layer7/top/vertical",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "http_method": http_method,
                        "http_version": http_version,
                        "ip_version": ip_version,
                        "limit": limit,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                    },
                    top_vertical_params.TopVerticalParams,
                ),
                post_parser=ResultWrapper[TopVerticalResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopVerticalResponse], ResultWrapper[TopVerticalResponse]),
        )


class AsyncTopResource(AsyncAPIResource):
    @cached_property
    def locations(self) -> AsyncLocationsResource:
        return AsyncLocationsResource(self._client)

    @cached_property
    def ases(self) -> AsyncAsesResource:
        return AsyncAsesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTopResourceWithStreamingResponse(self)

    async def attacks(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        limit_direction: Literal["ORIGIN", "TARGET"] | Omit = omit,
        limit_per_location: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN_MAX"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopAttacksResponse:
        """Retrieves the top attacks from origin to target location.

        Values are percentages
        of the total layer 7 attacks (with billing country). The attack magnitude can be
        defined by the number of mitigated requests or by the number of zones affected.
        You can optionally limit the number of attacks by origin/target location (useful
        if all the top attacks are from or to the same location).

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          limit_direction: Specifies whether the `limitPerLocation` applies to the source or target
              location.

          limit_per_location: Limits the number of attacks per origin/target (refer to `limitDirection`
              parameter) location.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/attacks/layer7/top/attacks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "limit_direction": limit_direction,
                        "limit_per_location": limit_per_location,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                        "normalization": normalization,
                    },
                    top_attacks_params.TopAttacksParams,
                ),
                post_parser=ResultWrapper[TopAttacksResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopAttacksResponse], ResultWrapper[TopAttacksResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar Attacks Layer 7 Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/attacks/subresources/layer7/methods/summary_v2/) instead."
    )
    async def industry(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        http_method: List[
            Literal[
                "GET",
                "POST",
                "DELETE",
                "PUT",
                "HEAD",
                "PURGE",
                "OPTIONS",
                "PROPFIND",
                "MKCOL",
                "PATCH",
                "ACL",
                "BCOPY",
                "BDELETE",
                "BMOVE",
                "BPROPFIND",
                "BPROPPATCH",
                "CHECKIN",
                "CHECKOUT",
                "CONNECT",
                "COPY",
                "LABEL",
                "LOCK",
                "MERGE",
                "MKACTIVITY",
                "MKWORKSPACE",
                "MOVE",
                "NOTIFY",
                "ORDERPATCH",
                "POLL",
                "PROPPATCH",
                "REPORT",
                "SEARCH",
                "SUBSCRIBE",
                "TRACE",
                "UNCHECKOUT",
                "UNLOCK",
                "UNSUBSCRIBE",
                "UPDATE",
                "VERSIONCONTROL",
                "BASELINECONTROL",
                "XMSENUMATTS",
                "RPC_OUT_DATA",
                "RPC_IN_DATA",
                "JSON",
                "COOK",
                "TRACK",
            ]
        ]
        | Omit = omit,
        http_version: List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]] | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopIndustryResponse:
        """This endpoint is deprecated.

        To continue getting this data, switch to the
        summary by industry endpoint.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          http_method: Filters results by HTTP method.

          http_version: Filters results by HTTP version.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/attacks/layer7/top/industry",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "http_method": http_method,
                        "http_version": http_version,
                        "ip_version": ip_version,
                        "limit": limit,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                    },
                    top_industry_params.TopIndustryParams,
                ),
                post_parser=ResultWrapper[TopIndustryResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopIndustryResponse], ResultWrapper[TopIndustryResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar Attacks Layer 7 Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/attacks/subresources/layer7/methods/summary_v2/) instead."
    )
    async def vertical(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        http_method: List[
            Literal[
                "GET",
                "POST",
                "DELETE",
                "PUT",
                "HEAD",
                "PURGE",
                "OPTIONS",
                "PROPFIND",
                "MKCOL",
                "PATCH",
                "ACL",
                "BCOPY",
                "BDELETE",
                "BMOVE",
                "BPROPFIND",
                "BPROPPATCH",
                "CHECKIN",
                "CHECKOUT",
                "CONNECT",
                "COPY",
                "LABEL",
                "LOCK",
                "MERGE",
                "MKACTIVITY",
                "MKWORKSPACE",
                "MOVE",
                "NOTIFY",
                "ORDERPATCH",
                "POLL",
                "PROPPATCH",
                "REPORT",
                "SEARCH",
                "SUBSCRIBE",
                "TRACE",
                "UNCHECKOUT",
                "UNLOCK",
                "UNSUBSCRIBE",
                "UPDATE",
                "VERSIONCONTROL",
                "BASELINECONTROL",
                "XMSENUMATTS",
                "RPC_OUT_DATA",
                "RPC_IN_DATA",
                "JSON",
                "COOK",
                "TRACK",
            ]
        ]
        | Omit = omit,
        http_version: List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]] | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        limit: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        mitigation_product: List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopVerticalResponse:
        """This endpoint is deprecated.

        To continue getting this data, switch to the
        summary by vertical endpoint.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          http_method: Filters results by HTTP method.

          http_version: Filters results by HTTP version.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          limit: Limits the number of objects returned in the response.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          mitigation_product: Filters the results by layer 7 mitigation product.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/attacks/layer7/top/vertical",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "http_method": http_method,
                        "http_version": http_version,
                        "ip_version": ip_version,
                        "limit": limit,
                        "location": location,
                        "mitigation_product": mitigation_product,
                        "name": name,
                    },
                    top_vertical_params.TopVerticalParams,
                ),
                post_parser=ResultWrapper[TopVerticalResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopVerticalResponse], ResultWrapper[TopVerticalResponse]),
        )


class TopResourceWithRawResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.attacks = to_raw_response_wrapper(
            top.attacks,
        )
        self.industry = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                top.industry,  # pyright: ignore[reportDeprecated],
            )
        )
        self.vertical = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                top.vertical,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def locations(self) -> LocationsResourceWithRawResponse:
        return LocationsResourceWithRawResponse(self._top.locations)

    @cached_property
    def ases(self) -> AsesResourceWithRawResponse:
        return AsesResourceWithRawResponse(self._top.ases)


class AsyncTopResourceWithRawResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.attacks = async_to_raw_response_wrapper(
            top.attacks,
        )
        self.industry = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                top.industry,  # pyright: ignore[reportDeprecated],
            )
        )
        self.vertical = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                top.vertical,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithRawResponse:
        return AsyncLocationsResourceWithRawResponse(self._top.locations)

    @cached_property
    def ases(self) -> AsyncAsesResourceWithRawResponse:
        return AsyncAsesResourceWithRawResponse(self._top.ases)


class TopResourceWithStreamingResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.attacks = to_streamed_response_wrapper(
            top.attacks,
        )
        self.industry = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                top.industry,  # pyright: ignore[reportDeprecated],
            )
        )
        self.vertical = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                top.vertical,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def locations(self) -> LocationsResourceWithStreamingResponse:
        return LocationsResourceWithStreamingResponse(self._top.locations)

    @cached_property
    def ases(self) -> AsesResourceWithStreamingResponse:
        return AsesResourceWithStreamingResponse(self._top.ases)


class AsyncTopResourceWithStreamingResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.attacks = async_to_streamed_response_wrapper(
            top.attacks,
        )
        self.industry = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                top.industry,  # pyright: ignore[reportDeprecated],
            )
        )
        self.vertical = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                top.vertical,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithStreamingResponse:
        return AsyncLocationsResourceWithStreamingResponse(self._top.locations)

    @cached_property
    def ases(self) -> AsyncAsesResourceWithStreamingResponse:
        return AsyncAsesResourceWithStreamingResponse(self._top.ases)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .action import Action
from .matcher import Matcher
from ..._models import BaseModel

__all__ = ["EmailRoutingRule"]


class EmailRoutingRule(BaseModel):
    id: Optional[str] = None
    """Routing rule identifier."""

    actions: Optional[List[Action]] = None
    """List actions patterns."""

    enabled: Optional[Literal[True, False]] = None
    """Routing rule status."""

    matchers: Optional[List[Matcher]] = None
    """Matching patterns to forward to your actions."""

    name: Optional[str] = None
    """Routing rule name."""

    priority: Optional[float] = None
    """Priority of the routing rule."""

    tag: Optional[str] = None
    """Routing rule tag. (Deprecated, replaced by routing rule identifier)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.networks import (
    hostname_route_edit_params,
    hostname_route_list_params,
    hostname_route_create_params,
)
from ....types.zero_trust.networks.hostname_route import HostnameRoute

__all__ = ["HostnameRoutesResource", "AsyncHostnameRoutesResource"]


class HostnameRoutesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HostnameRoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return HostnameRoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HostnameRoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return HostnameRoutesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        comment: str | Omit = omit,
        hostname: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Create a hostname route.

        Args:
          account_id: Cloudflare account ID

          comment: An optional description of the hostname route.

          hostname: The hostname of the route.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/zerotrust/routes/hostname", account_id=account_id),
            body=maybe_transform(
                {
                    "comment": comment,
                    "hostname": hostname,
                    "tunnel_id": tunnel_id,
                },
                hostname_route_create_params.HostnameRouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        hostname: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[HostnameRoute]:
        """
        Lists and filters hostname routes in an account.

        Args:
          account_id: Cloudflare account ID

          id: The hostname route ID.

          comment: If set, only list hostname routes with the given comment.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          hostname: If set, only list hostname routes that contain a substring of the given value,
              the filter is case-insensitive.

          is_deleted: If `true`, only return deleted hostname routes. If `false`, exclude deleted
              hostname routes.

          page: Page number of paginated results.

          per_page: Number of results to display.

          tunnel_id: If set, only list hostname routes that point to a specific tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/zerotrust/routes/hostname", account_id=account_id),
            page=SyncV4PagePaginationArray[HostnameRoute],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "comment": comment,
                        "existed_at": existed_at,
                        "hostname": hostname,
                        "is_deleted": is_deleted,
                        "page": page,
                        "per_page": per_page,
                        "tunnel_id": tunnel_id,
                    },
                    hostname_route_list_params.HostnameRouteListParams,
                ),
            ),
            model=HostnameRoute,
        )

    def delete(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Delete a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    def edit(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        hostname: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Updates a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          comment: An optional description of the hostname route.

          hostname: The hostname of the route.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            body=maybe_transform(
                {
                    "comment": comment,
                    "hostname": hostname,
                    "tunnel_id": tunnel_id,
                },
                hostname_route_edit_params.HostnameRouteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    def get(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Get a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )


class AsyncHostnameRoutesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHostnameRoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHostnameRoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHostnameRoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncHostnameRoutesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        comment: str | Omit = omit,
        hostname: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Create a hostname route.

        Args:
          account_id: Cloudflare account ID

          comment: An optional description of the hostname route.

          hostname: The hostname of the route.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/zerotrust/routes/hostname", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "hostname": hostname,
                    "tunnel_id": tunnel_id,
                },
                hostname_route_create_params.HostnameRouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        hostname: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[HostnameRoute, AsyncV4PagePaginationArray[HostnameRoute]]:
        """
        Lists and filters hostname routes in an account.

        Args:
          account_id: Cloudflare account ID

          id: The hostname route ID.

          comment: If set, only list hostname routes with the given comment.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          hostname: If set, only list hostname routes that contain a substring of the given value,
              the filter is case-insensitive.

          is_deleted: If `true`, only return deleted hostname routes. If `false`, exclude deleted
              hostname routes.

          page: Page number of paginated results.

          per_page: Number of results to display.

          tunnel_id: If set, only list hostname routes that point to a specific tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/zerotrust/routes/hostname", account_id=account_id),
            page=AsyncV4PagePaginationArray[HostnameRoute],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "comment": comment,
                        "existed_at": existed_at,
                        "hostname": hostname,
                        "is_deleted": is_deleted,
                        "page": page,
                        "per_page": per_page,
                        "tunnel_id": tunnel_id,
                    },
                    hostname_route_list_params.HostnameRouteListParams,
                ),
            ),
            model=HostnameRoute,
        )

    async def delete(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Delete a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    async def edit(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        hostname: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Updates a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          comment: An optional description of the hostname route.

          hostname: The hostname of the route.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "hostname": hostname,
                    "tunnel_id": tunnel_id,
                },
                hostname_route_edit_params.HostnameRouteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )

    async def get(
        self,
        hostname_route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HostnameRoute:
        """
        Get a hostname route.

        Args:
          account_id: Cloudflare account ID

          hostname_route_id: The hostname route ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not hostname_route_id:
            raise ValueError(f"Expected a non-empty value for `hostname_route_id` but received {hostname_route_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/zerotrust/routes/hostname/{hostname_route_id}",
                account_id=account_id,
                hostname_route_id=hostname_route_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[HostnameRoute]._unwrapper,
            ),
            cast_to=cast(Type[HostnameRoute], ResultWrapper[HostnameRoute]),
        )


class HostnameRoutesResourceWithRawResponse:
    def __init__(self, hostname_routes: HostnameRoutesResource) -> None:
        self._hostname_routes = hostname_routes

        self.create = to_raw_response_wrapper(
            hostname_routes.create,
        )
        self.list = to_raw_response_wrapper(
            hostname_routes.list,
        )
        self.delete = to_raw_response_wrapper(
            hostname_routes.delete,
        )
        self.edit = to_raw_response_wrapper(
            hostname_routes.edit,
        )
        self.get = to_raw_response_wrapper(
            hostname_routes.get,
        )


class AsyncHostnameRoutesResourceWithRawResponse:
    def __init__(self, hostname_routes: AsyncHostnameRoutesResource) -> None:
        self._hostname_routes = hostname_routes

        self.create = async_to_raw_response_wrapper(
            hostname_routes.create,
        )
        self.list = async_to_raw_response_wrapper(
            hostname_routes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            hostname_routes.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            hostname_routes.edit,
        )
        self.get = async_to_raw_response_wrapper(
            hostname_routes.get,
        )


class HostnameRoutesResourceWithStreamingResponse:
    def __init__(self, hostname_routes: HostnameRoutesResource) -> None:
        self._hostname_routes = hostname_routes

        self.create = to_streamed_response_wrapper(
            hostname_routes.create,
        )
        self.list = to_streamed_response_wrapper(
            hostname_routes.list,
        )
        self.delete = to_streamed_response_wrapper(
            hostname_routes.delete,
        )
        self.edit = to_streamed_response_wrapper(
            hostname_routes.edit,
        )
        self.get = to_streamed_response_wrapper(
            hostname_routes.get,
        )


class AsyncHostnameRoutesResourceWithStreamingResponse:
    def __init__(self, hostname_routes: AsyncHostnameRoutesResource) -> None:
        self._hostname_routes = hostname_routes

        self.create = async_to_streamed_response_wrapper(
            hostname_routes.create,
        )
        self.list = async_to_streamed_response_wrapper(
            hostname_routes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            hostname_routes.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            hostname_routes.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            hostname_routes.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .target_list_params import TargetListParams as TargetListParams
from .target_get_response import TargetGetResponse as TargetGetResponse
from .target_create_params import TargetCreateParams as TargetCreateParams
from .target_list_response import TargetListResponse as TargetListResponse
from .target_update_params import TargetUpdateParams as TargetUpdateParams
from .target_create_response import TargetCreateResponse as TargetCreateResponse
from .target_update_response import TargetUpdateResponse as TargetUpdateResponse
from .target_bulk_update_params import TargetBulkUpdateParams as TargetBulkUpdateParams
from .target_bulk_update_response import TargetBulkUpdateResponse as TargetBulkUpdateResponse
from .target_bulk_delete_v2_params import TargetBulkDeleteV2Params as TargetBulkDeleteV2Params



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["BodyScanningSettingsParam"]


class BodyScanningSettingsParam(TypedDict, total=False):
    """Specify the DLP inspection mode."""

    inspection_mode: Literal["deep", "shallow"]
    """Specify the inspection mode as either `deep` or `shallow`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...types.cache import (
    origin_cloud_region_list_params,
    origin_cloud_region_update_params,
    origin_cloud_region_bulk_update_params,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.cache.origin_cloud_region import OriginCloudRegion
from ...types.cache.origin_cloud_region_delete_response import OriginCloudRegionDeleteResponse
from ...types.cache.origin_cloud_region_bulk_delete_response import OriginCloudRegionBulkDeleteResponse
from ...types.cache.origin_cloud_region_bulk_update_response import OriginCloudRegionBulkUpdateResponse
from ...types.cache.origin_cloud_region_supported_regions_response import OriginCloudRegionSupportedRegionsResponse

__all__ = ["OriginCloudRegionsResource", "AsyncOriginCloudRegionsResource"]


class OriginCloudRegionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OriginCloudRegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OriginCloudRegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OriginCloudRegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OriginCloudRegionsResourceWithStreamingResponse(self)

    def update(
        self,
        path_origin_ip: str,
        *,
        zone_id: str,
        body_origin_ip: str,
        region: str,
        vendor: Literal["aws", "azure", "gcp", "oci"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegion]:
        """
        Creates a new IP-to-cloud-region mapping or replaces the existing mapping for
        the specified IP. PUT is idempotent — calling it repeatedly with the same body
        produces the same result. The IP path parameter is normalized to canonical form
        (RFC 5952 for IPv6) before storage. The vendor and region are validated against
        the list from `GET /zones/{zone_id}/origin/cloud_regions/supported_regions`.
        Returns 400 if the `origin_ip` in the body does not match the URL path
        parameter. Returns 403 (code 1164) when the zone has reached the limit of 3,500
        IP mappings.

        Args:
          zone_id: Identifier.

          body_origin_ip: Origin IP address (IPv4 or IPv6). For the single PUT endpoint
              (`PUT /origin/cloud_regions/{origin_ip}`), this field must match the path
              parameter or the request will be rejected with a 400 error. For the batch PUT
              endpoint, this field identifies which mapping to upsert.

          region: Cloud vendor region identifier. Must be a valid region for the specified vendor
              as returned by the supported_regions endpoint.

          vendor: Cloud vendor hosting the origin. Must be one of the supported vendors.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not path_origin_ip:
            raise ValueError(f"Expected a non-empty value for `path_origin_ip` but received {path_origin_ip!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/origin/cloud_regions/{path_origin_ip}", zone_id=zone_id, path_origin_ip=path_origin_ip
            ),
            body=maybe_transform(
                {
                    "body_origin_ip": body_origin_ip,
                    "region": region,
                    "vendor": vendor,
                },
                origin_cloud_region_update_params.OriginCloudRegionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCloudRegion]], ResultWrapper[OriginCloudRegion]),
        )

    def list(
        self,
        *,
        zone_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[OriginCloudRegion]:
        """
        Returns all IP-to-cloud-region mappings configured for the zone with pagination
        support. Each mapping tells Cloudflare which cloud vendor and region hosts the
        origin at that IP, enabling the edge to route via the nearest Tiered Cache
        upper-tier co-located with that cloud provider. Returns an empty array when no
        mappings exist.

        Args:
          zone_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin/cloud_regions", zone_id=zone_id),
            page=SyncV4PagePaginationArray[OriginCloudRegion],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    origin_cloud_region_list_params.OriginCloudRegionListParams,
                ),
            ),
            model=OriginCloudRegion,
        )

    def delete(
        self,
        origin_ip: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionDeleteResponse]:
        """Removes the cloud region mapping for a single origin IP address.

        The IP path
        parameter is normalized before lookup. Returns the deleted IP on success.
        Returns 404 if no mapping exists for the specified IP. When the last mapping for
        the zone is removed the underlying rule record is also deleted.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not origin_ip:
            raise ValueError(f"Expected a non-empty value for `origin_ip` but received {origin_ip!r}")
        return self._delete(
            path_template("/zones/{zone_id}/origin/cloud_regions/{origin_ip}", zone_id=zone_id, origin_ip=origin_ip),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionDeleteResponse]], ResultWrapper[OriginCloudRegionDeleteResponse]
            ),
        )

    def bulk_delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionBulkDeleteResponse]:
        """Removes up to 100 IP-to-cloud-region mappings in a single request.

        Each IP is
        validated independently — successfully deleted items are returned in the
        `succeeded` array and IPs that could not be found or are invalid are returned in
        the `failed` array.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/origin/cloud_regions/batch", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionBulkDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionBulkDeleteResponse]], ResultWrapper[OriginCloudRegionBulkDeleteResponse]
            ),
        )

    def bulk_update(
        self,
        *,
        zone_id: str,
        body: Iterable[origin_cloud_region_bulk_update_params.Body],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionBulkUpdateResponse]:
        """Upserts up to 100 IP-to-cloud-region mappings in a single request.

        Items in the
        request body are created or replaced; mappings not included in the request body
        are preserved unchanged (this is a merge operation, not a full collection
        replacement). Each item is validated independently — valid items are applied and
        invalid items are returned in the `failed` array. The vendor and region for
        every item are validated against the list from
        `GET /zones/{zone_id}/origin/cloud_regions/supported_regions`.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/origin/cloud_regions/batch", zone_id=zone_id),
            body=maybe_transform(body, Iterable[origin_cloud_region_bulk_update_params.Body]),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionBulkUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionBulkUpdateResponse]], ResultWrapper[OriginCloudRegionBulkUpdateResponse]
            ),
        )

    def get(
        self,
        origin_ip: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegion]:
        """Returns the cloud region mapping for a single origin IP address.

        The IP path
        parameter is normalized before lookup (RFC 5952 for IPv6). Returns 404 if the
        zone has no mappings or if the specified IP has no mapping.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not origin_ip:
            raise ValueError(f"Expected a non-empty value for `origin_ip` but received {origin_ip!r}")
        return self._get(
            path_template("/zones/{zone_id}/origin/cloud_regions/{origin_ip}", zone_id=zone_id, origin_ip=origin_ip),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCloudRegion]], ResultWrapper[OriginCloudRegion]),
        )

    def supported_regions(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionSupportedRegionsResponse]:
        """
        Returns the cloud vendors and regions that are valid values for origin cloud
        region mappings. Each region includes the Tiered Cache upper-tier colocation
        codes that will be used for cache routing when a mapping targeting that region
        is active. Requires the zone to have Tiered Cache enabled.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/origin/cloud_regions/supported_regions", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionSupportedRegionsResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionSupportedRegionsResponse]],
                ResultWrapper[OriginCloudRegionSupportedRegionsResponse],
            ),
        )


class AsyncOriginCloudRegionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOriginCloudRegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOriginCloudRegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOriginCloudRegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOriginCloudRegionsResourceWithStreamingResponse(self)

    async def update(
        self,
        path_origin_ip: str,
        *,
        zone_id: str,
        body_origin_ip: str,
        region: str,
        vendor: Literal["aws", "azure", "gcp", "oci"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegion]:
        """
        Creates a new IP-to-cloud-region mapping or replaces the existing mapping for
        the specified IP. PUT is idempotent — calling it repeatedly with the same body
        produces the same result. The IP path parameter is normalized to canonical form
        (RFC 5952 for IPv6) before storage. The vendor and region are validated against
        the list from `GET /zones/{zone_id}/origin/cloud_regions/supported_regions`.
        Returns 400 if the `origin_ip` in the body does not match the URL path
        parameter. Returns 403 (code 1164) when the zone has reached the limit of 3,500
        IP mappings.

        Args:
          zone_id: Identifier.

          body_origin_ip: Origin IP address (IPv4 or IPv6). For the single PUT endpoint
              (`PUT /origin/cloud_regions/{origin_ip}`), this field must match the path
              parameter or the request will be rejected with a 400 error. For the batch PUT
              endpoint, this field identifies which mapping to upsert.

          region: Cloud vendor region identifier. Must be a valid region for the specified vendor
              as returned by the supported_regions endpoint.

          vendor: Cloud vendor hosting the origin. Must be one of the supported vendors.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not path_origin_ip:
            raise ValueError(f"Expected a non-empty value for `path_origin_ip` but received {path_origin_ip!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/origin/cloud_regions/{path_origin_ip}", zone_id=zone_id, path_origin_ip=path_origin_ip
            ),
            body=await async_maybe_transform(
                {
                    "body_origin_ip": body_origin_ip,
                    "region": region,
                    "vendor": vendor,
                },
                origin_cloud_region_update_params.OriginCloudRegionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCloudRegion]], ResultWrapper[OriginCloudRegion]),
        )

    def list(
        self,
        *,
        zone_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OriginCloudRegion, AsyncV4PagePaginationArray[OriginCloudRegion]]:
        """
        Returns all IP-to-cloud-region mappings configured for the zone with pagination
        support. Each mapping tells Cloudflare which cloud vendor and region hosts the
        origin at that IP, enabling the edge to route via the nearest Tiered Cache
        upper-tier co-located with that cloud provider. Returns an empty array when no
        mappings exist.

        Args:
          zone_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin/cloud_regions", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[OriginCloudRegion],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    origin_cloud_region_list_params.OriginCloudRegionListParams,
                ),
            ),
            model=OriginCloudRegion,
        )

    async def delete(
        self,
        origin_ip: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionDeleteResponse]:
        """Removes the cloud region mapping for a single origin IP address.

        The IP path
        parameter is normalized before lookup. Returns the deleted IP on success.
        Returns 404 if no mapping exists for the specified IP. When the last mapping for
        the zone is removed the underlying rule record is also deleted.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not origin_ip:
            raise ValueError(f"Expected a non-empty value for `origin_ip` but received {origin_ip!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/origin/cloud_regions/{origin_ip}", zone_id=zone_id, origin_ip=origin_ip),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionDeleteResponse]], ResultWrapper[OriginCloudRegionDeleteResponse]
            ),
        )

    async def bulk_delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionBulkDeleteResponse]:
        """Removes up to 100 IP-to-cloud-region mappings in a single request.

        Each IP is
        validated independently — successfully deleted items are returned in the
        `succeeded` array and IPs that could not be found or are invalid are returned in
        the `failed` array.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/origin/cloud_regions/batch", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionBulkDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionBulkDeleteResponse]], ResultWrapper[OriginCloudRegionBulkDeleteResponse]
            ),
        )

    async def bulk_update(
        self,
        *,
        zone_id: str,
        body: Iterable[origin_cloud_region_bulk_update_params.Body],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionBulkUpdateResponse]:
        """Upserts up to 100 IP-to-cloud-region mappings in a single request.

        Items in the
        request body are created or replaced; mappings not included in the request body
        are preserved unchanged (this is a merge operation, not a full collection
        replacement). Each item is validated independently — valid items are applied and
        invalid items are returned in the `failed` array. The vendor and region for
        every item are validated against the list from
        `GET /zones/{zone_id}/origin/cloud_regions/supported_regions`.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/origin/cloud_regions/batch", zone_id=zone_id),
            body=await async_maybe_transform(body, Iterable[origin_cloud_region_bulk_update_params.Body]),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionBulkUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionBulkUpdateResponse]], ResultWrapper[OriginCloudRegionBulkUpdateResponse]
            ),
        )

    async def get(
        self,
        origin_ip: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegion]:
        """Returns the cloud region mapping for a single origin IP address.

        The IP path
        parameter is normalized before lookup (RFC 5952 for IPv6). Returns 404 if the
        zone has no mappings or if the specified IP has no mapping.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not origin_ip:
            raise ValueError(f"Expected a non-empty value for `origin_ip` but received {origin_ip!r}")
        return await self._get(
            path_template("/zones/{zone_id}/origin/cloud_regions/{origin_ip}", zone_id=zone_id, origin_ip=origin_ip),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegion]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OriginCloudRegion]], ResultWrapper[OriginCloudRegion]),
        )

    async def supported_regions(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OriginCloudRegionSupportedRegionsResponse]:
        """
        Returns the cloud vendors and regions that are valid values for origin cloud
        region mappings. Each region includes the Tiered Cache upper-tier colocation
        codes that will be used for cache routing when a mapping targeting that region
        is active. Requires the zone to have Tiered Cache enabled.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/origin/cloud_regions/supported_regions", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OriginCloudRegionSupportedRegionsResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OriginCloudRegionSupportedRegionsResponse]],
                ResultWrapper[OriginCloudRegionSupportedRegionsResponse],
            ),
        )


class OriginCloudRegionsResourceWithRawResponse:
    def __init__(self, origin_cloud_regions: OriginCloudRegionsResource) -> None:
        self._origin_cloud_regions = origin_cloud_regions

        self.update = to_raw_response_wrapper(
            origin_cloud_regions.update,
        )
        self.list = to_raw_response_wrapper(
            origin_cloud_regions.list,
        )
        self.delete = to_raw_response_wrapper(
            origin_cloud_regions.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            origin_cloud_regions.bulk_delete,
        )
        self.bulk_update = to_raw_response_wrapper(
            origin_cloud_regions.bulk_update,
        )
        self.get = to_raw_response_wrapper(
            origin_cloud_regions.get,
        )
        self.supported_regions = to_raw_response_wrapper(
            origin_cloud_regions.supported_regions,
        )


class AsyncOriginCloudRegionsResourceWithRawResponse:
    def __init__(self, origin_cloud_regions: AsyncOriginCloudRegionsResource) -> None:
        self._origin_cloud_regions = origin_cloud_regions

        self.update = async_to_raw_response_wrapper(
            origin_cloud_regions.update,
        )
        self.list = async_to_raw_response_wrapper(
            origin_cloud_regions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            origin_cloud_regions.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            origin_cloud_regions.bulk_delete,
        )
        self.bulk_update = async_to_raw_response_wrapper(
            origin_cloud_regions.bulk_update,
        )
        self.get = async_to_raw_response_wrapper(
            origin_cloud_regions.get,
        )
        self.supported_regions = async_to_raw_response_wrapper(
            origin_cloud_regions.supported_regions,
        )


class OriginCloudRegionsResourceWithStreamingResponse:
    def __init__(self, origin_cloud_regions: OriginCloudRegionsResource) -> None:
        self._origin_cloud_regions = origin_cloud_regions

        self.update = to_streamed_response_wrapper(
            origin_cloud_regions.update,
        )
        self.list = to_streamed_response_wrapper(
            origin_cloud_regions.list,
        )
        self.delete = to_streamed_response_wrapper(
            origin_cloud_regions.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            origin_cloud_regions.bulk_delete,
        )
        self.bulk_update = to_streamed_response_wrapper(
            origin_cloud_regions.bulk_update,
        )
        self.get = to_streamed_response_wrapper(
            origin_cloud_regions.get,
        )
        self.supported_regions = to_streamed_response_wrapper(
            origin_cloud_regions.supported_regions,
        )


class AsyncOriginCloudRegionsResourceWithStreamingResponse:
    def __init__(self, origin_cloud_regions: AsyncOriginCloudRegionsResource) -> None:
        self._origin_cloud_regions = origin_cloud_regions

        self.update = async_to_streamed_response_wrapper(
            origin_cloud_regions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            origin_cloud_regions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            origin_cloud_regions.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            origin_cloud_regions.bulk_delete,
        )
        self.bulk_update = async_to_streamed_response_wrapper(
            origin_cloud_regions.bulk_update,
        )
        self.get = async_to_streamed_response_wrapper(
            origin_cloud_regions.get,
        )
        self.supported_regions = async_to_streamed_response_wrapper(
            origin_cloud_regions.supported_regions,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TokenVerifyResponse"]


class TokenVerifyResponse(BaseModel):
    id: str
    """Token identifier tag."""

    status: Literal["active", "disabled", "expired"]
    """Status of the token."""

    expires_on: Optional[datetime] = None
    """
    The expiration time on or after which the JWT MUST NOT be accepted for
    processing.
    """

    not_before: Optional[datetime] = None
    """The time before which the token MUST NOT be accepted for processing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["SessionAffinityAttributesParam"]


class SessionAffinityAttributesParam(TypedDict, total=False):
    """Configures attributes for session affinity."""

    drain_duration: float
    """Configures the drain duration in seconds.

    This field is only used when session affinity is enabled on the load balancer.
    """

    headers: SequenceNotStr[str]
    """
    Configures the names of HTTP headers to base session affinity on when header
    `session_affinity` is enabled. At least one HTTP header name must be provided.
    To specify the exact cookies to be used, include an item in the following
    format: `"cookie:<cookie-name-1>,<cookie-name-2>"` (example) where everything
    after the colon is a comma-separated list of cookie names. Providing only
    `"cookie"` will result in all cookies being used. The default max number of HTTP
    header names that can be provided depends on your plan: 5 for Enterprise, 1 for
    all other plans.
    """

    require_all_headers: bool
    """
    When header `session_affinity` is enabled, this option can be used to specify
    how HTTP headers on load balancing requests will be used. The supported values
    are: - `"true"`: Load balancing requests must contain _all_ of the HTTP headers
    specified by the `headers` session affinity attribute, otherwise sessions aren't
    created. - `"false"`: Load balancing requests must contain _at least one_ of the
    HTTP headers specified by the `headers` session affinity attribute, otherwise
    sessions aren't created.
    """

    samesite: Literal["Auto", "Lax", "None", "Strict"]
    """Configures the SameSite attribute on session affinity cookie.

    Value "Auto" will be translated to "Lax" or "None" depending if Always Use HTTPS
    is enabled. Note: when using value "None", the secure attribute can not be set
    to "Never".
    """

    secure: Literal["Auto", "Always", "Never"]
    """Configures the Secure attribute on session affinity cookie.

    Value "Always" indicates the Secure attribute will be set in the Set-Cookie
    header, "Never" indicates the Secure attribute will not be set, and "Auto" will
    set the Secure attribute depending if Always Use HTTPS is enabled.
    """

    zero_downtime_failover: Literal["none", "temporary", "sticky"]
    """
    Configures the zero-downtime failover between origins within a pool when session
    affinity is enabled. This feature is currently incompatible with Argo, Tiered
    Cache, and Bandwidth Alliance. The supported values are: - `"none"`: No failover
    takes place for sessions pinned to the origin (default). - `"temporary"`:
    Traffic will be sent to another other healthy origin until the originally pinned
    origin is available; note that this can potentially result in heavy origin
    flapping. - `"sticky"`: The session affinity cookie is updated and subsequent
    requests are sent to the new origin. Note: Zero-downtime failover with sticky
    sessions is currently not supported for session affinity by header.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .group import Group as Group
from .waf_rule_group import WAFRuleGroup as WAFRuleGroup
from .rule_edit_params import RuleEditParams as RuleEditParams
from .rule_list_params import RuleListParams as RuleListParams
from .group_edit_params import GroupEditParams as GroupEditParams
from .group_list_params import GroupListParams as GroupListParams
from .rule_get_response import RuleGetResponse as RuleGetResponse
from .group_get_response import GroupGetResponse as GroupGetResponse
from .rule_edit_response import RuleEditResponse as RuleEditResponse
from .rule_list_response import RuleListResponse as RuleListResponse
from .group_edit_response import GroupEditResponse as GroupEditResponse
from .allowed_modes_anomaly import AllowedModesAnomaly as AllowedModesAnomaly



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from .packages import (
    PackagesResource,
    AsyncPackagesResource,
    PackagesResourceWithRawResponse,
    AsyncPackagesResourceWithRawResponse,
    PackagesResourceWithStreamingResponse,
    AsyncPackagesResourceWithStreamingResponse,
)

__all__ = [
    "GroupsResource",
    "AsyncGroupsResource",
    "GroupsResourceWithRawResponse",
    "AsyncGroupsResourceWithRawResponse",
    "GroupsResourceWithStreamingResponse",
    "AsyncGroupsResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "PackagesResource",
    "AsyncPackagesResource",
    "PackagesResourceWithRawResponse",
    "AsyncPackagesResourceWithRawResponse",
    "PackagesResourceWithStreamingResponse",
    "AsyncPackagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .labels import (
    LabelsResource,
    AsyncLabelsResource,
    LabelsResourceWithRawResponse,
    AsyncLabelsResourceWithRawResponse,
    LabelsResourceWithStreamingResponse,
    AsyncLabelsResourceWithStreamingResponse,
)
from .schemas import (
    SchemasResource,
    AsyncSchemasResource,
    SchemasResourceWithRawResponse,
    AsyncSchemasResourceWithRawResponse,
    SchemasResourceWithStreamingResponse,
    AsyncSchemasResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .discovery import (
    DiscoveryResource,
    AsyncDiscoveryResource,
    DiscoveryResourceWithRawResponse,
    AsyncDiscoveryResourceWithRawResponse,
    DiscoveryResourceWithStreamingResponse,
    AsyncDiscoveryResourceWithStreamingResponse,
)
from .operations import (
    OperationsResource,
    AsyncOperationsResource,
    OperationsResourceWithRawResponse,
    AsyncOperationsResourceWithRawResponse,
    OperationsResourceWithStreamingResponse,
    AsyncOperationsResourceWithStreamingResponse,
)
from .api_gateway import (
    APIGatewayResource,
    AsyncAPIGatewayResource,
    APIGatewayResourceWithRawResponse,
    AsyncAPIGatewayResourceWithRawResponse,
    APIGatewayResourceWithStreamingResponse,
    AsyncAPIGatewayResourceWithStreamingResponse,
)
from .user_schemas import (
    UserSchemasResource,
    AsyncUserSchemasResource,
    UserSchemasResourceWithRawResponse,
    AsyncUserSchemasResourceWithRawResponse,
    UserSchemasResourceWithStreamingResponse,
    AsyncUserSchemasResourceWithStreamingResponse,
)
from .configurations import (
    ConfigurationsResource,
    AsyncConfigurationsResource,
    ConfigurationsResourceWithRawResponse,
    AsyncConfigurationsResourceWithRawResponse,
    ConfigurationsResourceWithStreamingResponse,
    AsyncConfigurationsResourceWithStreamingResponse,
)
from .expression_template import (
    ExpressionTemplateResource,
    AsyncExpressionTemplateResource,
    ExpressionTemplateResourceWithRawResponse,
    AsyncExpressionTemplateResourceWithRawResponse,
    ExpressionTemplateResourceWithStreamingResponse,
    AsyncExpressionTemplateResourceWithStreamingResponse,
)

__all__ = [
    "ConfigurationsResource",
    "AsyncConfigurationsResource",
    "ConfigurationsResourceWithRawResponse",
    "AsyncConfigurationsResourceWithRawResponse",
    "ConfigurationsResourceWithStreamingResponse",
    "AsyncConfigurationsResourceWithStreamingResponse",
    "DiscoveryResource",
    "AsyncDiscoveryResource",
    "DiscoveryResourceWithRawResponse",
    "AsyncDiscoveryResourceWithRawResponse",
    "DiscoveryResourceWithStreamingResponse",
    "AsyncDiscoveryResourceWithStreamingResponse",
    "LabelsResource",
    "AsyncLabelsResource",
    "LabelsResourceWithRawResponse",
    "AsyncLabelsResourceWithRawResponse",
    "LabelsResourceWithStreamingResponse",
    "AsyncLabelsResourceWithStreamingResponse",
    "OperationsResource",
    "AsyncOperationsResource",
    "OperationsResourceWithRawResponse",
    "AsyncOperationsResourceWithRawResponse",
    "OperationsResourceWithStreamingResponse",
    "AsyncOperationsResourceWithStreamingResponse",
    "SchemasResource",
    "AsyncSchemasResource",
    "SchemasResourceWithRawResponse",
    "AsyncSchemasResourceWithRawResponse",
    "SchemasResourceWithStreamingResponse",
    "AsyncSchemasResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "UserSchemasResource",
    "AsyncUserSchemasResource",
    "UserSchemasResourceWithRawResponse",
    "AsyncUserSchemasResourceWithRawResponse",
    "UserSchemasResourceWithStreamingResponse",
    "AsyncUserSchemasResourceWithStreamingResponse",
    "ExpressionTemplateResource",
    "AsyncExpressionTemplateResource",
    "ExpressionTemplateResourceWithRawResponse",
    "AsyncExpressionTemplateResourceWithRawResponse",
    "ExpressionTemplateResourceWithStreamingResponse",
    "AsyncExpressionTemplateResourceWithStreamingResponse",
    "APIGatewayResource",
    "AsyncAPIGatewayResource",
    "APIGatewayResourceWithRawResponse",
    "AsyncAPIGatewayResourceWithRawResponse",
    "APIGatewayResourceWithStreamingResponse",
    "AsyncAPIGatewayResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["AccountUpdateResponse", "Error", "ErrorSource", "Message", "MessageSource", "ResultInfo"]


class ErrorSource(BaseModel):
    pointer: Optional[str] = None


class Error(BaseModel):
    code: int

    message: str

    documentation_url: Optional[str] = None

    source: Optional[ErrorSource] = None


class MessageSource(BaseModel):
    pointer: Optional[str] = None


class Message(BaseModel):
    code: int

    message: str

    documentation_url: Optional[str] = None

    source: Optional[MessageSource] = None


class ResultInfo(BaseModel):
    count: Optional[float] = None
    """Total number of results for the requested service."""

    page: Optional[float] = None
    """Current page within paginated list of results."""

    per_page: Optional[float] = None
    """Number of results per page of results."""

    total_count: Optional[float] = None
    """Total results available without any search parameters."""

    total_pages: Optional[float] = None
    """The number of total pages in the entire result set."""


class AccountUpdateResponse(BaseModel):
    errors: List[Error]

    messages: List[Message]

    success: Literal[True]
    """Whether the API call was successful."""

    result_info: Optional[ResultInfo] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.access.logs.scim import update_list_params
from ......types.zero_trust.access.logs.scim.update_list_response import UpdateListResponse

__all__ = ["UpdatesResource", "AsyncUpdatesResource"]


class UpdatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UpdatesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        idp_id: SequenceNotStr[str],
        cf_resource_id: SequenceNotStr[str] | Omit = omit,
        direction: Literal["desc", "asc"] | Omit = omit,
        idp_resource_id: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        request_method: List[Literal["DELETE", "PATCH", "POST", "PUT"]] | Omit = omit,
        resource_group_name: SequenceNotStr[str] | Omit = omit,
        resource_type: List[Literal["USER", "GROUP"]] | Omit = omit,
        resource_user_email: SequenceNotStr[str] | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        status: List[Literal["FAILURE", "SUCCESS"]] | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[UpdateListResponse]:
        """
        Lists Access SCIM update logs that maintain a record of updates made to User and
        Group resources synced to Cloudflare via the System for Cross-domain Identity
        Management (SCIM).

        Args:
          account_id: Identifier.

          idp_id: The unique Id of the IdP that has SCIM enabled.

          cf_resource_id: The unique Cloudflare-generated Id of the SCIM resource. Pass once for a single
              lookup (`?cf_resource_id=A`) or repeat the parameter
              (`?cf_resource_id=A&cf_resource_id=B`) to filter by multiple resources in one
              request.

          direction: The chronological order used to sort the logs.

          idp_resource_id: The IdP-generated Id of the SCIM resource. Pass once for a single lookup
              (`?idp_resource_id=A`) or repeat the parameter
              (`?idp_resource_id=A&idp_resource_id=B`) to filter by multiple resources in one
              request.

          limit: The maximum number of update logs to retrieve.

          page: Page number of results.

          per_page: Number of results per page.

          request_method: The request method of the SCIM request.

          resource_group_name: The display name of the SCIM Group resource. Pass once for a single lookup
              (`?resource_group_name=A`) or repeat the parameter
              (`?resource_group_name=A&resource_group_name=B`) to filter by multiple group
              names in one request.

          resource_type: The resource type of the SCIM request.

          resource_user_email: The email address of the SCIM User resource. Pass once for a single lookup
              (`?resource_user_email=A`) or repeat the parameter
              (`?resource_user_email=A&resource_user_email=B`) to filter by multiple emails in
              one request.

          since: the timestamp of the earliest update log.

          status: The status of the SCIM request.

          until: the timestamp of the most-recent update log.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/logs/scim/updates", account_id=account_id),
            page=SyncV4PagePaginationArray[UpdateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "idp_id": idp_id,
                        "cf_resource_id": cf_resource_id,
                        "direction": direction,
                        "idp_resource_id": idp_resource_id,
                        "limit": limit,
                        "page": page,
                        "per_page": per_page,
                        "request_method": request_method,
                        "resource_group_name": resource_group_name,
                        "resource_type": resource_type,
                        "resource_user_email": resource_user_email,
                        "since": since,
                        "status": status,
                        "until": until,
                    },
                    update_list_params.UpdateListParams,
                ),
            ),
            model=UpdateListResponse,
        )


class AsyncUpdatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUpdatesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        idp_id: SequenceNotStr[str],
        cf_resource_id: SequenceNotStr[str] | Omit = omit,
        direction: Literal["desc", "asc"] | Omit = omit,
        idp_resource_id: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        request_method: List[Literal["DELETE", "PATCH", "POST", "PUT"]] | Omit = omit,
        resource_group_name: SequenceNotStr[str] | Omit = omit,
        resource_type: List[Literal["USER", "GROUP"]] | Omit = omit,
        resource_user_email: SequenceNotStr[str] | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        status: List[Literal["FAILURE", "SUCCESS"]] | Omit = omit,
        until: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UpdateListResponse, AsyncV4PagePaginationArray[UpdateListResponse]]:
        """
        Lists Access SCIM update logs that maintain a record of updates made to User and
        Group resources synced to Cloudflare via the System for Cross-domain Identity
        Management (SCIM).

        Args:
          account_id: Identifier.

          idp_id: The unique Id of the IdP that has SCIM enabled.

          cf_resource_id: The unique Cloudflare-generated Id of the SCIM resource. Pass once for a single
              lookup (`?cf_resource_id=A`) or repeat the parameter
              (`?cf_resource_id=A&cf_resource_id=B`) to filter by multiple resources in one
              request.

          direction: The chronological order used to sort the logs.

          idp_resource_id: The IdP-generated Id of the SCIM resource. Pass once for a single lookup
              (`?idp_resource_id=A`) or repeat the parameter
              (`?idp_resource_id=A&idp_resource_id=B`) to filter by multiple resources in one
              request.

          limit: The maximum number of update logs to retrieve.

          page: Page number of results.

          per_page: Number of results per page.

          request_method: The request method of the SCIM request.

          resource_group_name: The display name of the SCIM Group resource. Pass once for a single lookup
              (`?resource_group_name=A`) or repeat the parameter
              (`?resource_group_name=A&resource_group_name=B`) to filter by multiple group
              names in one request.

          resource_type: The resource type of the SCIM request.

          resource_user_email: The email address of the SCIM User resource. Pass once for a single lookup
              (`?resource_user_email=A`) or repeat the parameter
              (`?resource_user_email=A&resource_user_email=B`) to filter by multiple emails in
              one request.

          since: the timestamp of the earliest update log.

          status: The status of the SCIM request.

          until: the timestamp of the most-recent update log.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/logs/scim/updates", account_id=account_id),
            page=AsyncV4PagePaginationArray[UpdateListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "idp_id": idp_id,
                        "cf_resource_id": cf_resource_id,
                        "direction": direction,
                        "idp_resource_id": idp_resource_id,
                        "limit": limit,
                        "page": page,
                        "per_page": per_page,
                        "request_method": request_method,
                        "resource_group_name": resource_group_name,
                        "resource_type": resource_type,
                        "resource_user_email": resource_user_email,
                        "since": since,
                        "status": status,
                        "until": until,
                    },
                    update_list_params.UpdateListParams,
                ),
            ),
            model=UpdateListResponse,
        )


class UpdatesResourceWithRawResponse:
    def __init__(self, updates: UpdatesResource) -> None:
        self._updates = updates

        self.list = to_raw_response_wrapper(
            updates.list,
        )


class AsyncUpdatesResourceWithRawResponse:
    def __init__(self, updates: AsyncUpdatesResource) -> None:
        self._updates = updates

        self.list = async_to_raw_response_wrapper(
            updates.list,
        )


class UpdatesResourceWithStreamingResponse:
    def __init__(self, updates: UpdatesResource) -> None:
        self._updates = updates

        self.list = to_streamed_response_wrapper(
            updates.list,
        )


class AsyncUpdatesResourceWithStreamingResponse:
    def __init__(self, updates: AsyncUpdatesResource) -> None:
        self._updates = updates

        self.list = async_to_streamed_response_wrapper(
            updates.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.dex.tests import unique_device_list_params
from .....types.zero_trust.dex.tests.unique_devices import UniqueDevices

__all__ = ["UniqueDevicesResource", "AsyncUniqueDevicesResource"]


class UniqueDevicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UniqueDevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UniqueDevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UniqueDevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UniqueDevicesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        device_id: SequenceNotStr[str] | Omit = omit,
        test_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UniqueDevices]:
        """
        Returns unique count of devices that have run synthetic application monitoring
        tests in the past 7 days.

        Args:
          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          test_name: Optionally filter results by test name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dex/tests/unique-devices", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "device_id": device_id,
                        "test_name": test_name,
                    },
                    unique_device_list_params.UniqueDeviceListParams,
                ),
                post_parser=ResultWrapper[Optional[UniqueDevices]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UniqueDevices]], ResultWrapper[UniqueDevices]),
        )


class AsyncUniqueDevicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUniqueDevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUniqueDevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUniqueDevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUniqueDevicesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        device_id: SequenceNotStr[str] | Omit = omit,
        test_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UniqueDevices]:
        """
        Returns unique count of devices that have run synthetic application monitoring
        tests in the past 7 days.

        Args:
          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          test_name: Optionally filter results by test name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dex/tests/unique-devices", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "device_id": device_id,
                        "test_name": test_name,
                    },
                    unique_device_list_params.UniqueDeviceListParams,
                ),
                post_parser=ResultWrapper[Optional[UniqueDevices]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UniqueDevices]], ResultWrapper[UniqueDevices]),
        )


class UniqueDevicesResourceWithRawResponse:
    def __init__(self, unique_devices: UniqueDevicesResource) -> None:
        self._unique_devices = unique_devices

        self.list = to_raw_response_wrapper(
            unique_devices.list,
        )


class AsyncUniqueDevicesResourceWithRawResponse:
    def __init__(self, unique_devices: AsyncUniqueDevicesResource) -> None:
        self._unique_devices = unique_devices

        self.list = async_to_raw_response_wrapper(
            unique_devices.list,
        )


class UniqueDevicesResourceWithStreamingResponse:
    def __init__(self, unique_devices: UniqueDevicesResource) -> None:
        self._unique_devices = unique_devices

        self.list = to_streamed_response_wrapper(
            unique_devices.list,
        )


class AsyncUniqueDevicesResourceWithStreamingResponse:
    def __init__(self, unique_devices: AsyncUniqueDevicesResource) -> None:
        self._unique_devices = unique_devices

        self.list = async_to_streamed_response_wrapper(
            unique_devices.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .latest import (
    LatestResource,
    AsyncLatestResource,
    LatestResourceWithRawResponse,
    AsyncLatestResourceWithRawResponse,
    LatestResourceWithStreamingResponse,
    AsyncLatestResourceWithStreamingResponse,
)

__all__ = [
    "LatestResource",
    "AsyncLatestResource",
    "LatestResourceWithRawResponse",
    "AsyncLatestResourceWithRawResponse",
    "LatestResourceWithStreamingResponse",
    "AsyncLatestResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WAF"]


class WAF(BaseModel):
    id: Optional[Literal["waf"]] = None
    """
    Turn on or off
    [WAF managed rules (previous version, deprecated)](https://developers.cloudflare.com/waf/reference/legacy/old-waf-managed-rules/).
    You cannot enable or disable individual WAF managed rules via Page Rules.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of WAF managed rules (previous version)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CatalogSyncUpdateParams"]


class CatalogSyncUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    description: str

    name: str

    policy: str

    update_mode: Literal["AUTO", "MANUAL"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResourceListResponse"]


class ResourceListResponse(BaseModel):
    id: str
    """Share Resource identifier."""

    created: datetime
    """When the share was created."""

    meta: object
    """Resource Metadata."""

    modified: datetime
    """When the share was modified."""

    resource_account_id: str
    """Account identifier."""

    resource_id: str
    """Share Resource identifier."""

    resource_type: Literal[
        "custom-ruleset",
        "gateway-policy",
        "gateway-destination-ip",
        "gateway-block-page-settings",
        "gateway-extended-email-matching",
        "idp-federation-grant",
    ]
    """Resource Type."""

    resource_version: int
    """Resource Version."""

    status: Literal["active", "deleting", "deleted"]
    """Resource Status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TopupStatusParams"]


class TopupStatusParams(TypedDict, total=False):
    account_id: Required[str]

    payment_intent_id: Required[str]
    """Stripe invoice ID to check status for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from ....._types import SequenceNotStr
from .pattern_param import PatternParam
from ..context_awareness_param import ContextAwarenessParam

__all__ = [
    "CustomCreateParams",
    "Entry",
    "EntryDLPNewCustomEntry",
    "EntryDLPNewWordListEntry",
    "SensitivityLevel",
    "SharedEntry",
]


class CustomCreateParams(TypedDict, total=False):
    account_id: Required[str]

    name: Required[str]

    ai_context_enabled: bool

    allowed_match_count: int
    """Related DLP policies will trigger when the match count exceeds the number set."""

    confidence_threshold: Optional[str]

    context_awareness: ContextAwarenessParam
    """
    Scan the context of predefined entries to only return matches surrounded by
    keywords.
    """

    data_classes: SequenceNotStr[str]
    """Data class IDs to associate with the profile."""

    data_tags: SequenceNotStr[str]
    """Data tag IDs to associate with the profile."""

    description: Optional[str]
    """The description of the profile."""

    entries: Iterable[Entry]

    ocr_enabled: bool

    sensitivity_levels: Iterable[SensitivityLevel]
    """Sensitivity levels to associate with the profile."""

    shared_entries: Iterable[SharedEntry]
    """Entries from other profiles (e.g.

    pre-defined Cloudflare profiles, or your Microsoft Information Protection
    profiles).
    """


class EntryDLPNewCustomEntry(TypedDict, total=False):
    enabled: Required[bool]

    name: Required[str]

    pattern: Required[PatternParam]

    description: Optional[str]


class EntryDLPNewWordListEntry(TypedDict, total=False):
    enabled: Required[bool]

    name: Required[str]

    words: Required[SequenceNotStr[str]]


Entry: TypeAlias = Union[EntryDLPNewCustomEntry, EntryDLPNewWordListEntry]


class SensitivityLevel(TypedDict, total=False):
    """
    A reference pairing a sensitivity group with a specific level within that group.
    """

    group_id: Required[str]

    level_id: Required[str]


class SharedEntry(TypedDict, total=False):
    enabled: Required[bool]

    entry_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .nat import Nat
from ...._models import BaseModel
from .routed_subnet import RoutedSubnet
from .lan_static_addressing import LANStaticAddressing

__all__ = ["LAN"]


class LAN(BaseModel):
    id: Optional[str] = None
    """Identifier"""

    bond_id: Optional[int] = None

    ha_link: Optional[bool] = None
    """mark true to use this LAN for HA probing.

    only works for site with HA turned on. only one LAN can be set as the ha_link.
    """

    is_breakout: Optional[bool] = None
    """mark true to use this LAN for source-based breakout traffic"""

    is_prioritized: Optional[bool] = None
    """mark true to use this LAN for source-based prioritized traffic"""

    name: Optional[str] = None

    nat: Optional[Nat] = None

    physport: Optional[int] = None

    routed_subnets: Optional[List[RoutedSubnet]] = None

    site_id: Optional[str] = None
    """Identifier"""

    static_addressing: Optional[LANStaticAddressing] = None
    """
    If the site is not configured in high availability mode, this configuration is
    optional (if omitted, use DHCP). However, if in high availability mode,
    static_address is required along with secondary and virtual address.
    """

    vlan_tag: Optional[int] = None
    """VLAN ID. Use zero for untagged."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["OriginCACertificateDeleteResponse"]


class OriginCACertificateDeleteResponse(BaseModel):
    id: Optional[str] = None
    """Identifier."""

    revoked_at: Optional[datetime] = None
    """When the certificate was revoked."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.registrar.workflow_status import WorkflowStatus

__all__ = ["UpdateStatusResource", "AsyncUpdateStatusResource"]


class UpdateStatusResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UpdateStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UpdateStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UpdateStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UpdateStatusResourceWithStreamingResponse(self)

    def get(
        self,
        domain_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowStatus:
        """
        Returns the current status of a domain update workflow.

        Use this endpoint to poll for completion when the PATCH response returned
        `202 Accepted`. The URL is provided in the `links.self` field of the workflow
        status response.

        Poll this endpoint until the workflow reaches a terminal state or a state that
        requires user attention.

        Use increasing backoff between polls. When the workflow remains blocked on a
        third party, use a longer polling interval and do not poll indefinitely.

        Args:
          account_id: Identifier

          domain_name: Fully qualified domain name (FQDN) including the extension (e.g., `example.com`,
              `mybrand.app`). The domain name uniquely identifies a registration — the same
              domain cannot be registered twice, making it a natural idempotency key for
              registration requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not domain_name:
            raise ValueError(f"Expected a non-empty value for `domain_name` but received {domain_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/registrar/registrations/{domain_name}/update-status",
                account_id=account_id,
                domain_name=domain_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowStatus]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowStatus], ResultWrapper[WorkflowStatus]),
        )


class AsyncUpdateStatusResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUpdateStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUpdateStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUpdateStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUpdateStatusResourceWithStreamingResponse(self)

    async def get(
        self,
        domain_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowStatus:
        """
        Returns the current status of a domain update workflow.

        Use this endpoint to poll for completion when the PATCH response returned
        `202 Accepted`. The URL is provided in the `links.self` field of the workflow
        status response.

        Poll this endpoint until the workflow reaches a terminal state or a state that
        requires user attention.

        Use increasing backoff between polls. When the workflow remains blocked on a
        third party, use a longer polling interval and do not poll indefinitely.

        Args:
          account_id: Identifier

          domain_name: Fully qualified domain name (FQDN) including the extension (e.g., `example.com`,
              `mybrand.app`). The domain name uniquely identifies a registration — the same
              domain cannot be registered twice, making it a natural idempotency key for
              registration requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not domain_name:
            raise ValueError(f"Expected a non-empty value for `domain_name` but received {domain_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/registrar/registrations/{domain_name}/update-status",
                account_id=account_id,
                domain_name=domain_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowStatus]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowStatus], ResultWrapper[WorkflowStatus]),
        )


class UpdateStatusResourceWithRawResponse:
    def __init__(self, update_status: UpdateStatusResource) -> None:
        self._update_status = update_status

        self.get = to_raw_response_wrapper(
            update_status.get,
        )


class AsyncUpdateStatusResourceWithRawResponse:
    def __init__(self, update_status: AsyncUpdateStatusResource) -> None:
        self._update_status = update_status

        self.get = async_to_raw_response_wrapper(
            update_status.get,
        )


class UpdateStatusResourceWithStreamingResponse:
    def __init__(self, update_status: UpdateStatusResource) -> None:
        self._update_status = update_status

        self.get = to_streamed_response_wrapper(
            update_status.get,
        )


class AsyncUpdateStatusResourceWithStreamingResponse:
    def __init__(self, update_status: AsyncUpdateStatusResource) -> None:
        self._update_status = update_status

        self.get = async_to_streamed_response_wrapper(
            update_status.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tests import (
    TestsResource,
    AsyncTestsResource,
    TestsResourceWithRawResponse,
    AsyncTestsResourceWithRawResponse,
    TestsResourceWithStreamingResponse,
    AsyncTestsResourceWithStreamingResponse,
)
from .unique_devices import (
    UniqueDevicesResource,
    AsyncUniqueDevicesResource,
    UniqueDevicesResourceWithRawResponse,
    AsyncUniqueDevicesResourceWithRawResponse,
    UniqueDevicesResourceWithStreamingResponse,
    AsyncUniqueDevicesResourceWithStreamingResponse,
)

__all__ = [
    "UniqueDevicesResource",
    "AsyncUniqueDevicesResource",
    "UniqueDevicesResourceWithRawResponse",
    "AsyncUniqueDevicesResourceWithRawResponse",
    "UniqueDevicesResourceWithStreamingResponse",
    "AsyncUniqueDevicesResourceWithStreamingResponse",
    "TestsResource",
    "AsyncTestsResource",
    "TestsResourceWithRawResponse",
    "AsyncTestsResourceWithRawResponse",
    "TestsResourceWithStreamingResponse",
    "AsyncTestsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .origin import Origin
from ..._models import BaseModel
from .check_region import CheckRegion
from .load_shedding import LoadShedding
from .origin_steering import OriginSteering
from .notification_filter import NotificationFilter

__all__ = ["Pool"]


class Pool(BaseModel):
    id: Optional[str] = None

    check_regions: Optional[List[CheckRegion]] = None
    """A list of regions from which to run health checks.

    Null means every Cloudflare data center.
    """

    created_on: Optional[str] = None

    description: Optional[str] = None
    """A human-readable description of the pool."""

    disabled_at: Optional[datetime] = None
    """This field shows up only if the pool is disabled.

    This field is set with the time the pool was disabled at.
    """

    enabled: Optional[bool] = None
    """Whether to enable (the default) or disable this pool.

    Disabled pools will not receive traffic and are excluded from health checks.
    Disabling a pool will cause any load balancers using it to failover to the next
    pool (if any).
    """

    latitude: Optional[float] = None
    """
    The latitude of the data center containing the origins used in this pool in
    decimal degrees. If this is set, longitude must also be set.
    """

    load_shedding: Optional[LoadShedding] = None
    """Configures load shedding policies and percentages for the pool."""

    longitude: Optional[float] = None
    """
    The longitude of the data center containing the origins used in this pool in
    decimal degrees. If this is set, latitude must also be set.
    """

    minimum_origins: Optional[int] = None
    """
    The minimum number of origins that must be healthy for this pool to serve
    traffic. If the number of healthy origins falls below this number, the pool will
    be marked unhealthy and will failover to the next available pool.
    """

    modified_on: Optional[str] = None

    monitor: Optional[str] = None
    """
    The ID of the Monitor to use for checking the health of origins within this
    pool.
    """

    monitor_group: Optional[str] = None
    """
    The ID of the Monitor Group to use for checking the health of origins within
    this pool.
    """

    name: Optional[str] = None
    """A short name (tag) for the pool.

    Only alphanumeric characters, hyphens, and underscores are allowed.
    """

    networks: Optional[List[str]] = None
    """List of networks where Load Balancer or Pool is enabled."""

    notification_email: Optional[str] = None
    """This field is now deprecated.

    It has been moved to Cloudflare's Centralized Notification service
    https://developers.cloudflare.com/fundamentals/notifications/. The email address
    to send health status notifications to. This can be an individual mailbox or a
    mailing list. Multiple emails can be supplied as a comma delimited list.
    """

    notification_filter: Optional[NotificationFilter] = None
    """Filter pool and origin health notifications by resource type or health status.

    Use null to reset.
    """

    origin_steering: Optional[OriginSteering] = None
    """Configures origin steering for the pool.

    Controls how origins are selected for new sessions and traffic without session
    affinity.
    """

    origins: Optional[List[Origin]] = None
    """The list of origins within this pool.

    Traffic directed at this pool is balanced across all currently healthy origins,
    provided the pool itself is healthy.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .wan_static_addressing_param import WANStaticAddressingParam

__all__ = ["WANCreateParams"]


class WANCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    physport: Required[int]

    name: str

    priority: int

    static_addressing: WANStaticAddressingParam
    """(optional) if omitted, use DHCP.

    Submit secondary_address when site is in high availability mode.
    """

    vlan_tag: int
    """VLAN ID. Use zero for untagged."""
